@@ -3,9 +3,9 @@ use std::{convert::TryFrom, num::NonZeroU16, time::Duration};
 
 use futures::{future::ok, FutureExt, SinkExt, StreamExt};
 use ntex::codec::Framed;
-use ntex::rt::time::delay_for;
+use ntex::rt::time::sleep;
 use ntex::server;
-use ntex::util::{ByteString, Bytes};
+use ntex::util::{poll_fn, ByteString, Bytes};
 
 use ntex_mqtt::v5::{
     client, codec, error, ControlMessage, Handshake, HandshakeAck, MqttServer, Publish,
@@ -49,9 +49,6 @@ async fn handshake<Io>(packet: Handshake<Io>) -> Result<HandshakeAck<Io, St>, Te
 
 #[ntex::test]
 async fn test_simple() -> std::io::Result<()> {
-    std::env::set_var("RUST_LOG", "ntex_mqtt=trace,ntex_codec=info,ntex=trace");
-    env_logger::init();
-
     let srv = server::test_server(|| {
         MqttServer::new(handshake).publish(|p: Publish| ok::<_, TestError>(p.ack())).finish()
     });
@@ -80,7 +77,7 @@ async fn test_disconnect() -> std::io::Result<()> {
                 ok::<_, TestError>(ntex::fn_service(move |p: Publish| {
                     session.sink().close();
                     async move {
-                        delay_for(Duration::from_millis(100)).await;
+                        sleep(Duration::from_millis(100)).await;
                         Ok::<_, TestError>(p.ack())
                     }
                 }))
@@ -115,7 +112,7 @@ async fn test_disconnect_with_reason() -> std::io::Result<()> {
                     };
                     session.sink().close_with_reason(pkt);
                     async move {
-                        delay_for(Duration::from_millis(100)).await;
+                        sleep(Duration::from_millis(100)).await;
                         Ok::<_, TestError>(p.ack())
                     }
                 }))
@@ -181,7 +178,7 @@ async fn test_ack_order() -> std::io::Result<()> {
     let srv = server::test_server(move || {
         MqttServer::new(handshake)
             .publish(|p: Publish| {
-                delay_for(Duration::from_millis(100)).map(move |_| Ok::<_, TestError>(p.ack()))
+                sleep(Duration::from_millis(100)).map(move |_| Ok::<_, TestError>(p.ack()))
             })
             .control(move |msg| match msg {
                 ControlMessage::Subscribe(mut msg) => {
@@ -263,8 +260,7 @@ async fn test_dups() {
     let srv = server::test_server(move || {
         MqttServer::new(handshake)
             .publish(|p: Publish| {
-                delay_for(Duration::from_millis(10000))
-                    .map(move |_| Ok::<_, TestError>(p.ack()))
+                sleep(Duration::from_millis(10000)).map(move |_| Ok::<_, TestError>(p.ack()))
             })
             .finish()
     });
@@ -377,8 +373,7 @@ async fn test_max_receive() {
             .receive_max(1)
             .max_qos(codec::QoS::AtLeastOnce)
             .publish(|p: Publish| {
-                delay_for(Duration::from_millis(10000))
-                    .map(move |_| Ok::<_, TestError>(p.ack()))
+                sleep(Duration::from_millis(10000)).map(move |_| Ok::<_, TestError>(p.ack()))
             })
             .control(move |msg| match msg {
                 ControlMessage::ProtocolError(msg) => ok::<_, TestError>(msg.ack()),
@@ -463,7 +458,7 @@ async fn test_keepalive() {
     ntex::rt::spawn(client.start_default());
 
     assert!(sink.is_open());
-    delay_for(Duration::from_millis(2500)).await;
+    sleep(Duration::from_millis(2500)).await;
     assert!(!sink.is_open());
     assert!(ka.load(Relaxed));
 }
@@ -502,11 +497,11 @@ async fn test_keepalive2() {
     let res =
         sink.publish(ByteString::from_static("#"), Bytes::new()).send_at_least_once().await;
     assert!(res.is_ok());
-    delay_for(Duration::from_millis(1200)).await;
+    sleep(Duration::from_millis(1200)).await;
     let res =
         sink.publish(ByteString::from_static("#"), Bytes::new()).send_at_least_once().await;
     assert!(res.is_ok());
-    delay_for(Duration::from_millis(2500)).await;
+    sleep(Duration::from_millis(2500)).await;
 
     assert!(!sink.is_open());
     assert!(ka.load(Relaxed));
@@ -532,7 +527,7 @@ async fn test_sink_encoder_error_pub_qos1() {
             Ok(con.ack(St))
         })
         .publish(|p: Publish| {
-            delay_for(Duration::from_millis(50)).map(move |_| Ok::<_, TestError>(p.ack()))
+            sleep(Duration::from_millis(50)).map(move |_| Ok::<_, TestError>(p.ack()))
         })
         .control(move |msg| match msg {
             ControlMessage::ProtocolError(msg) => ok::<_, TestError>(msg.ack()),
@@ -576,7 +571,7 @@ async fn test_sink_encoder_error_pub_qos0() {
             Ok(con.ack(St))
         })
         .publish(|p: Publish| {
-            delay_for(Duration::from_millis(50)).map(move |_| Ok::<_, TestError>(p.ack()))
+            sleep(Duration::from_millis(50)).map(move |_| Ok::<_, TestError>(p.ack()))
         })
         .control(move |msg| match msg {
             ControlMessage::ProtocolError(msg) => ok::<_, TestError>(msg.ack()),
@@ -697,3 +692,51 @@ async fn test_suback_with_reason() -> std::io::Result<()> {
 
     Ok(())
 }
+
+#[ntex::test]
+async fn test_handle_incoming() -> std::io::Result<()> {
+    let publish = Arc::new(AtomicBool::new(false));
+    let publish2 = publish.clone();
+    let disconnect = Arc::new(AtomicBool::new(false));
+    let disconnect2 = disconnect.clone();
+
+    let srv = server::test_server(move || {
+        let publish = publish2.clone();
+        let disconnect = disconnect2.clone();
+        MqttServer::new(handshake)
+            .publish(move |p: Publish| {
+                publish.store(true, Relaxed);
+                ok::<_, TestError>(p.ack())
+            })
+            .control(move |msg| match msg {
+                ControlMessage::Disconnect(msg) => {
+                    disconnect.store(true, Relaxed);
+                    ok::<_, TestError>(msg.ack())
+                }
+                _ => ok(msg.disconnect()),
+            })
+            .finish()
+    });
+
+    let io = srv.connect().await.unwrap();
+    let mut framed = Framed::new(io, codec::Codec::default());
+    framed.write(codec::Packet::Connect(codec::Connect::default().client_id("user"))).unwrap();
+    framed.write(pkt_publish().into()).unwrap();
+    framed
+        .write(codec::Packet::Disconnect(codec::Disconnect {
+            reason_code: codec::DisconnectReasonCode::ReceiveMaximumExceeded,
+            session_expiry_interval_secs: None,
+            server_reference: None,
+            reason_string: None,
+            user_properties: Default::default(),
+        }))
+        .unwrap();
+    poll_fn(|cx| framed.flush(cx)).await.unwrap();
+    drop(framed);
+    sleep(Duration::from_millis(500)).await;
+
+    assert!(publish.load(Relaxed));
+    assert!(disconnect.load(Relaxed));
+
+    Ok(())
+}
